package com.mycompany.commerce.rest.mycomponent.handler;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.util.logging.Logger;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;

import com.ibm.commerce.foundation.common.util.logging.LoggingHelper;
import com.ibm.commerce.foundation.rest.resourcehandler.OperationRegistry;
import com.ibm.commerce.rest.classic.core.AbstractConfigBasedClassicHandler;
import com.ibm.commerce.rest.javadoc.*;
import java.sql.Timestamp;
import com.mycompany.commerce.mycomponent.beans.MyResourceListDataBean;


import com.mycompany.commerce.mycomponent.commands.MyResourceCreateCmd;

import javax.ws.rs.core.MediaType;

/**
 * My resource.
 */
@Path("store/{storeId}/my_resource")
@Description("My resource.")
public class MyResourceHandler extends AbstractConfigBasedClassicHandler {

	private static final String CLASSNAME = MyResourceHandler.class.getName();
	private static final Logger LOGGER = LoggingHelper
			.getLogger(MyResourceHandler.class);

	private static final String RESOURCE_NAME = "my_resource";

	private static final String RESULT_LIST = "resultList";

	private static final String FIND_BY_MY_RESOURCE_ID_PATH = "{myResourceId}";
	private static final String FIND_BY_MY_RESOURCE_LIST_PATH = "myResourceListDataBean";

	private static final String SELF_QUERY_NAME = "self";
	private static final String SELF_QUERY_DESCRIPTION = "Finds my resources created by the current user.";
	private static final String SELF_METHOD_NAME = "findSelf";

	private static final String PARAMETER_MY_RESOURCE_ID = "myResourceId";
	private static final String PARAMETER_MY_RESOURCE_ID_DESCRIPTION = "My resource ID.";

	private static final String PARAMETER_FIELD1 = "field1";
	private static final String PARAMETER_FIELD1_DESCRIPTION = "Field1.";

	private static final String PARAMETER_FIELD2 = "field2";
	private static final String PARAMETER_FIELD2_DESCRIPTION = "Field2.";

	private static final String PROFILE_NAME_MY_COMPANY_DATABEANLIST_SUMMARY = "MyCompany_DatabeanList_Summary";
	private static final String PROFILE_NAME_MY_COMPANY_DATABEAN_SUMMARY = "MyCompany_Databean_Summary";

	private static final String PROFILE_NAME_MY_COMMAND_SUMMARY = "MyCommand_Summary";

	private static final String COMMAND_INTERFACE_NAME_PARAMETER = "com.mycompany.commerce.mycomponent.commands.MyResourceCreateCmd";
	private static final String LISTDATABEAN_CLASS_NAME_PARAMETER = "com.mycompany.commerce.mycomponent.beans.MyResourceListDataBean";
	private static final String BEAN_CLASS_NAME_PARAMETER = "com.mycompany.commerce.mycomponent.beans.MyResourceDataBean";

	private static final String FIND_BY_MY_RESOURCE_ID_DESCRIPTION = "Finds my resource by its ID.";
	private static final String FIND_BY_MY_RESOURCE_LIST_DESCRIPTION = "Finds my resource by My resource list.";
	private static final String FIND_MY_RESOURCE_BY_QUERY_DESCRIPTION = 
	    "Finds my resources by a query. See each query for details on input and output.";
	private static final String CREATE_ACTION_DESCRIPTION = "Creates my resource.";
	private static class ResourceCreateRequest {
		@ParameterDescription(description = "The content of field 1 for my resource .")
		private String field1;
		@ParameterDescription(description = "The content of field 2 for my resource.")
		private Timestamp field2;
	}

	private static class MyresourceCreateResponse {
		@ParameterDescription(description = "The ID of my resource.")
		private long myResourceId;
	}

	
	@Override
	public String getResourceName() {
		return RESOURCE_NAME;
	}

	/**
	 * Finds my resource by its ID.
	 * 
	 * @param storeId
	 *            The store ID.
	 * @param myResourceId
	 *            My resource ID.
	 * @return The response.
	 */
	@GET
	@Path(FIND_BY_MY_RESOURCE_ID_PATH)
	@Produces({ MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML,
			MediaType.APPLICATION_XHTML_XML, MediaType.APPLICATION_ATOM_XML })
	@Description(FIND_BY_MY_RESOURCE_ID_DESCRIPTION)
	@ResponseSchema(valueClass = MyResourceListDataBean.class, valueClassType = ClassType.Bean, 
	  profile = PROFILE_NAME_MY_COMPANY_DATABEAN_SUMMARY, responseCodes = {
	    @ResponseCode(code = 200, reason = RESPONSE_200_DESCRIPTION),
	    @ResponseCode(code = 400, reason = RESPONSE_400_DESCRIPTION),
	    @ResponseCode(code = 401, reason = RESPONSE_401_DESCRIPTION),
	    @ResponseCode(code = 403, reason = RESPONSE_403_DESCRIPTION),
	    @ResponseCode(code = 500, reason = RESPONSE_500_DESCRIPTION) })
	public Response findByMyResourceId(
			@PathParam(PARAMETER_STORE_ID)  @ParameterDescription(description = PARAMETER_STORE_ID_DESCRIPTION, 
			  valueProviderClass = StoreIdProvider.class, required = true) String storeId,
			@PathParam(PARAMETER_MY_RESOURCE_ID) String myResourceId,
			@QueryParam(value = "responseFormat") String responseFormat) {
		// Set up trace facilities
		final String METHODNAME = "findByMyResourceId(String storeId, String myResourceId, String responseFormat)";

		// Log method start
		final boolean traceEnabled = LoggingHelper.isTraceEnabled(LOGGER);
		final boolean entryExitTraceEnabled = LoggingHelper
				.isEntryExitTraceEnabled(LOGGER);
		if (entryExitTraceEnabled) {
			Object[] objArr = new Object[] { storeId, myResourceId };
			LOGGER.entering(CLASSNAME, METHODNAME, objArr);
		}

		/**
		 * Use the configuration-based REST API to automatically fill in input
		 * values, execute, and build the response.
		 */
		Response response = executeConfigBasedBeanWithContext(
				BEAN_CLASS_NAME_PARAMETER,
				PROFILE_NAME_MY_COMPANY_DATABEAN_SUMMARY, responseFormat, null);

		// Log method exit

		if (entryExitTraceEnabled) {
			LOGGER.exiting(CLASSNAME, METHODNAME, response);
		}
		return response;

	};

	/**
	 * Finds my resources.
	 * 
	 * @param storeId
	 *            The store ID.
	 * @return The response.
	 */
	@Description(FIND_BY_MY_RESOURCE_LIST_DESCRIPTION)
	@GET
	@Path(FIND_BY_MY_RESOURCE_LIST_PATH)
	@Produces({ MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML,
			MediaType.APPLICATION_XHTML_XML, MediaType.APPLICATION_ATOM_XML })
	public Response findMyResourceList(
			@QueryParam("responseFormat") String responseFormat,
			@PathParam("storeId") String storeId) {

		// Set up trace facilities
		final String METHODNAME = "findMyResourceList(String responseFormat,String storeId)";
		final boolean traceEnabled = LoggingHelper.isTraceEnabled(LOGGER);
		final boolean entryExitTraceEnabled = LoggingHelper
				.isEntryExitTraceEnabled(LOGGER);
		if (entryExitTraceEnabled) {
			Object[] parameters = new Object[] { responseFormat, storeId };
			LOGGER.entering(CLASSNAME, METHODNAME, parameters);
		}

		/**
		 * Use the configuration-based REST API to automatically fill in input
		 * values, execute, and build the response.
		 */
		Response result = executeConfigBasedBeanWithContext(
				LISTDATABEAN_CLASS_NAME_PARAMETER,
				PROFILE_NAME_MY_COMPANY_DATABEANLIST_SUMMARY, responseFormat,
				null);

		// Log method exit
		if (entryExitTraceEnabled) {
			LOGGER.exiting(CLASSNAME, METHODNAME, result);
		}

		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.ibm.commerce.foundation.rest.resourcehandler.AbstractBaseResourceHandler
	 * #
	 * initializeQueryRegistry(com.ibm.commerce.foundation.rest.resourcehandler
	 * .OperationRegistry)
	 */
	@Override
	public void initializeQueryRegistry(OperationRegistry registry) {
		registerOperationWithMethodSignature(registry, SELF_QUERY_NAME,
				SELF_QUERY_DESCRIPTION, SELF_METHOD_NAME, String.class);
	}

	/**
	 * Finds my resources by a query. See each query for details on input and
	 * output.
	 * 
	 * @param storeId
	 *            The store ID.
	 * @param queryName
	 *            The query name.
	 * @return The response.
	 */
	@Description(FIND_MY_RESOURCE_BY_QUERY_DESCRIPTION)
	@GET
	@Produces({ MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML,
			MediaType.APPLICATION_XHTML_XML, MediaType.APPLICATION_ATOM_XML })
	public Response findByQuery(@PathParam(PARAMETER_STORE_ID) @ParameterDescription(description = PARAMETER_STORE_ID_DESCRIPTION, 
			valueProviderClass = StoreIdProvider.class, required = true) String storeId,
			@QueryParam(PARAMETER_QUERY) String queryName) {

		final String METHODNAME = "findByQuery(String storeId, String queryName)";
		// Log method start
		final boolean traceEnabled = LoggingHelper.isTraceEnabled(LOGGER);
		final boolean entryExitTraceEnabled = LoggingHelper
				.isEntryExitTraceEnabled(LOGGER);
		if (entryExitTraceEnabled) {
			LOGGER.entering(CLASSNAME, METHODNAME, new Object[] { storeId,
					queryName });
		}

		String responseFormat = getParameterValue(PARAMETER_RESPONSE_FORMAT,
				null, false);

		Response response = prepareAndValidate(storeId, RESOURCE_NAME,
				GET_METHOD, request, responseFormat);
		if (response == null) {
			response = runQuery(queryName, responseFormat, storeId);
		}
		// Log method exit
		if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME, response);
		}
		return response;

	};

	/**
	 * Finds my resources created by the current user.
	 * 
	 * @param storeId
	 *            The store ID.
	 * @return The response.
	 */
	@InternalHttpMethod(GET_METHOD)
	@InternalProduces({ MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML,
			MediaType.APPLICATION_XHTML_XML, MediaType.APPLICATION_ATOM_XML })
	@Description(SELF_QUERY_DESCRIPTION)
	public Response findSelf(@PathParam(PARAMETER_STORE_ID) String storeId) {
		// Set up trace facilities
		final String METHODNAME = "findSelf(String storeId)";
		// Log method start
		final boolean traceEnabled = LoggingHelper.isTraceEnabled(LOGGER);
		final boolean entryExitTraceEnabled = LoggingHelper
				.isEntryExitTraceEnabled(LOGGER);
		if (entryExitTraceEnabled) {
			Object[] objArr = new Object[] { storeId };
			LOGGER.entering(CLASSNAME, METHODNAME, objArr);
		}

		String responseFormat = getParameterValue(PARAMETER_RESPONSE_FORMAT,
				null, false);

		/**
		 * Use the configuration-based REST API to automatically fill in input
		 * values, execute, and build the response.
		 */
		Response response = executeConfigBasedBeanWithContext(
				LISTDATABEAN_CLASS_NAME_PARAMETER,
				PROFILE_NAME_MY_COMPANY_DATABEANLIST_SUMMARY, responseFormat,
				null);

		// Log method exit

		if (entryExitTraceEnabled) {
			LOGGER.exiting(CLASSNAME, METHODNAME, response);
		}
		return response;

	}

	/**
	 * Creates my resource.
	 * 
	 * @param storeId
	 *            The store ID.
	 * @return The response.
	 */
	@POST
	@Description(CREATE_ACTION_DESCRIPTION)
	 @AdditionalParameterList({
	        @ParameterDescription(name = ParameterDescription.BODY_NAME, paramType = ParameterDescription.BODY_TYPE, 
	          description = ParameterDescription.BODY_DESCRIPTION, typeClass = ResourceCreateRequest.class) })
	    @ResponseSchema(valueClass = MyResourceCreateCmd.class, valueClassType = ClassType.Command, 
	      profile =  PROFILE_NAME_MY_COMMAND_SUMMARY,responseCodes = {
	    @ResponseCode(code = 200, reason = RESPONSE_200_DESCRIPTION),
	    @ResponseCode(code = 201, reason = RESPONSE_200_DESCRIPTION),
	    @ResponseCode(code = 400, reason = RESPONSE_400_DESCRIPTION),
	    @ResponseCode(code = 401, reason = RESPONSE_401_DESCRIPTION),
	    @ResponseCode(code = 403, reason = RESPONSE_403_DESCRIPTION),
	    @ResponseCode(code = 500, reason = RESPONSE_500_DESCRIPTION) })
	@Produces({ MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML,
			MediaType.APPLICATION_XHTML_XML, MediaType.APPLICATION_ATOM_XML })
	public Response create(@PathParam(PARAMETER_STORE_ID) @ParameterDescription(description = PARAMETER_STORE_ID_DESCRIPTION, 
	valueProviderClass = StoreIdProvider.class, required = true) String storeId,
	@QueryParam(value = "responseFormat") String responseFormat) {
		// Set up trace facilities
		final String METHODNAME = "create(String storeId, String responseFormat)";
		// Log method start
		final boolean traceEnabled = LoggingHelper.isTraceEnabled(LOGGER);
		final boolean entryExitTraceEnabled = LoggingHelper
				.isEntryExitTraceEnabled(LOGGER);
		if (entryExitTraceEnabled) {
			Object[] objArr = new Object[] { storeId, responseFormat };
			LOGGER.entering(CLASSNAME, METHODNAME, objArr);
		}
		/**
		 * Use the configuration-based REST API to automatically fill in input
		 * values, execute, and build the response.
		 */
		Response result = executeConfigBasedCommandWithContext(
				COMMAND_INTERFACE_NAME_PARAMETER,
				PROFILE_NAME_MY_COMMAND_SUMMARY, responseFormat, storeId, null);
		// Log method exit
		if (entryExitTraceEnabled) {
			LOGGER.exiting(CLASSNAME, METHODNAME, result);
		}

		return result;
	}

}
